#if !defined _PCLTA_
#define _PCLTA_

/* ------------------- */
/* masks for registers */
/* ------------------- */

#define SLEEPENG                0x20
#define DLPBA                   0x10
#define DLBA                    0x08
#define ULREADY                 0x04
#define RESET                   0x02
#define _READY                  0x01
    /* Status Register */

#define CLRIRQ                  0x01
    /* Interrupt Clear Register */

#define IRQ15                   0x20
#define IRQ12                   0x10
#define IRQ11                   0x08
#define IRQ10                   0x04
#define IRQ9                    0x02
#define IRQ5                    0x01
    /* IRQ Selection Register */

#define _RESET                  0x10
#define CLK_10                  0x00
#define CLK_5                   0x01
#define CLK_2_5                 0x02
#define CLK_1_25                0x03
#define CLK_0_625               0x04
    /* Clock and Reset Control Register */

#define IRQPLRTY                0x80
#define RESET_E                 0x40
#define DLREADY_E               0x20
#define DLPBA_E                 0x10
#define DLBA_E                  0x08
#define ULREADY_E               0x04
    /* Interrupt Reguest Control Register (E is for 'enable') */

#define TRANSCIEVER             0xf8
#define BATTERY                 0x03 /* only for PCNSS relevant */
    /* Transciever ID Register */

#define CMD_XFER                0x01
#define CMD_ACK                 0x02
    /* control bytes to transfer according to the protocol */

/* ---------------- */
/* register offsets */
/* ---------------- */

#define data_reg(dev)           ((dev)->io)        /* input/output */
#define status_reg(dev)         ((dev)->io+1)        /* input */
#define clrirq_reg(dev)         ((dev)->io+1)        /* output */
#define selirq_reg(dev)         ((dev)->io+2)        /* output */
#define clkrst_reg(dev)         ((dev)->io+3)        /* output */

#define _data_reg(n)            (n)                /* input/output */
#define _status_reg(n)          ((n)+1)                /* input */
#define _clrirq_reg(n)          ((n)+1)                /* output */
#define _selirq_reg(n)          ((n)+2)                /* output */
#define _clkrst_reg(n)          ((n)+3)                /* output */

/* --------------------------------------- */
/* communication protocol relevant defines */
/* --------------------------------------- */

#define NI_Q_CMD                0xf0
#define NI_QUEUE                0x0f
    /* masks to get network command and network buffer */

#define niCOMM                  0x10
#define niNETMGMT               0x20
#define niSERVICE               0xE6
    /* commands that need output buffer avaiable */

#define niRESET                 0x50
#define niFLUSH_CANCEL          0x60
#define niFLUSH_COMPLETE        0x60
#define niONLINE                0x70
#define niOFFLINE               0x80
#define niFLUSH                 0x90
#define niFLUSH_IGN             0xA0
#define niSLEEP                 0xB0
#define niSSTATUS               0xE0
#define niIRQENA                0xE5
    /* commands that do not need output buffer */

#define niTQ                    0x02
#define niTQ_P                  0x03
#define niNTQ                   0x04
#define niNTQ_P                 0x05
#define niPRIORITY              0x01 /* mask */
    /* output queues */

#define niRESPONSE              0x06
#define niINCOMING              0x08
    /* input queues */

/* ------------ */
/* boolean type */
/* ------------ */

typedef enum { false=0, true=1 } boolean;

/* --------------------------------- */
/* host buffers defines and typedefs */
/* --------------------------------- */

#define PIO_DATA_MAX            255        /* max. length of message */
#define BUF_PACKETS             10        /* size of buffers */

enum dev_state {
    NotOpened,
    IdleState,
    XferUnderway
};

struct xfer_packet {
    struct xfer_packet *next;
    boolean busy;
    unsigned char xfer_length;
    unsigned char xfer_cmd;
    unsigned char xfer_data[PIO_DATA_MAX];
};

struct pclta_buffer {
    struct xfer_packet *head, *tail;
    struct xfer_packet data[BUF_PACKETS];
};

struct pclta_device {
    unsigned int io;
    unsigned int irq;
    unsigned char clock;
    struct wait_queue *wait_queue;
    boolean restart_uplink;
    enum dev_state state;
    struct pclta_buffer *rx;
    struct pclta_buffer *tx;
};

/* -------------------------- */
/* error codes during ismod() */
/* -------------------------- */

#define PCLTA_IOPERM      1        /* I/O ports permission denied */
#define PCLTA_HSHK        2        /* hardware handshake failed */
#define PCLTA_LRESET      3        /* node did'n leave RESET state */
#define PCLTA_RESP        4        /* no response after reset */
#define PCLTA_IRQPERM     5        /* IRQ permission denied */
#define PCLTA_STAT        6        /* no response on status message */
#define PCLTA_TSCV        7        /* transciever not supported */
#define PCLTA_MAJOR       8        /* major number permission denied */
#define PCLTA_CDEV        9        /* create device failure */

/* ---------------- */
/* useful constants */
/* ---------------- */

static unsigned char irq_mask[16] =
    { 0,0,0,0,0,IRQ5,0,0,0,IRQ9,IRQ10,IRQ11,IRQ12,0,0,IRQ15};
    /* interrupt mask table for setting up the irq register */

static char *transceiver[32] =
{
    "", "TP/XF-78", "", "TP/XF-1250",
    "FT-10", "TP-RS485-39", "", "RF-10",
    "", "PL-10", "TP-RS485-625", "TP-RS485-1250",
    "TP-RS485-78", "", "", "",
    "PL-20C", "PL-20N", "PL-30", "",
    "", "", "", "",
    "FO-10", "", "", "DC-78",
    "DC-625", "DC-1250", "", ""
};
    /* supported transceiver types */

static unsigned char clock[32] =
{
    CLK_10, CLK_10, CLK_10, CLK_10,
    CLK_10, CLK_10, CLK_10, CLK_5,
    CLK_10, CLK_10, CLK_10, CLK_10,
    CLK_10, CLK_10, CLK_10, CLK_10,
    CLK_10, CLK_10, CLK_10, CLK_10,
    CLK_10, CLK_10, CLK_10, CLK_10,
    CLK_10, CLK_10, CLK_10, CLK_10,
    CLK_10, CLK_10, CLK_10, CLK_10,
};
    /* clock speeds that should be set up, depending on the transceiver type */

/* ------------------ */
/* for debug purposes */
/* ------------------ */

#if defined DEBUG
#define PCLTA_DEBUG(arg) arg
#else
#define PCLTA_DEBUG(arg)
#endif

#endif
